==> bndlib-0.0.160.jar: OSGI-OPT/src/aQute/lib/osgi/Processor.java <==
/* Copyright 2006 aQute SARL 
 * Licensed under the Apache License, Version 2.0, see http://www.apache.org/licenses/LICENSE-2.0 */
package aQute.lib.osgi;

import java.io.*;
import java.util.*;
import java.util.regex.*;

import aQute.lib.osgi.header.*;
import aQute.lib.reporter.*;

public class Processor implements Reporter {
	public final static String	DEFAULT_BND_EXTENSION	= ".bnd";
	public final static String	DEFAULT_JAR_EXTENSION	= ".jar";
	public final static String	DEFAULT_BAR_EXTENSION	= ".bar";

	List						errors					= new ArrayList();
	List						warnings				= new ArrayList();
	List						progress				= new ArrayList();
	boolean						pedantic;

	public void getInfo(Reporter processor) {
		errors.addAll(processor.getErrors());
		warnings.addAll(processor.getWarnings());
		progress.addAll(processor.getProgress());
		pedantic = processor.isPedantic();
	}

	public void warning(String string) {
		warnings.add(string);
	}

	public void error(String string) {
		errors.add(string);
	}

	public List getWarnings() {
		return warnings;
	}

	public List getErrors() {
		return errors;
	}

	public Map parseHeader(String value) {
		return parseHeader(value, this);
	}

	/**
	 * Standard OSGi header parser.
	 * 
	 * @param value
	 * @return
	 */
	static public Map parseHeader(String value, Processor logger) {
		return OSGiHeader.parseHeader(value,logger);
	}

	public Map analyzeBundleClasspath(Jar dot, Map bundleClasspath,
			Map contained, Map referred, Map uses) throws IOException {
		Map classSpace = new HashMap();

		if (bundleClasspath.isEmpty()) {
			analyzeJar(dot, "", classSpace, contained, referred, uses);
		} else {
			for (Iterator j = bundleClasspath.keySet().iterator(); j.hasNext();) {
				String path = (String) j.next();
				if (path.equals(".")) {
					analyzeJar(dot, "", classSpace, contained, referred, uses);
					continue;
				}
				//
				// There are 3 cases:
				// - embedded JAR file
				// - directory
				// - error
				//

				Resource resource = dot.getResource(path);
				if (resource != null) {
					try {
						Jar jar = new Jar(path);
						EmbeddedResource.build(jar, resource);
						analyzeJar(jar, "", classSpace, contained, referred,
								uses);
					} catch (Exception e) {
						warning("Invalid bundle classpath entry: " + path + " "
								+ e);
					}
				} else {
					if (dot.getDirectories().containsKey(path)) {
						analyzeJar(dot, path, classSpace, contained, referred,
								uses);
					} else {
						warning("No sub JAR or directory " + path);
					}
				}
			}
		}
		return classSpace;
	}

	/**
	 * We traverse through al the classes that we can find and calculate the
	 * contained and referred set and uses. This method ignores the Bundle
	 * classpath.
	 * 
	 * @param jar
	 * @param contained
	 * @param referred
	 * @param uses
	 * @throws IOException
	 */
	private void analyzeJar(Jar jar, String prefix, Map classSpace,
			Map contained, Map referred, Map uses) throws IOException {
		next: for (Iterator r = jar.getResources().keySet().iterator(); r
				.hasNext();) {
			String path = (String) r.next();
			if (path.startsWith(prefix)) {
				String relativePath = path.substring(prefix.length());
				String pack = getPackage(relativePath);

				if (!contained.containsKey(pack)) {
					if (!(pack.equals(".") || pack.equals("META-INF"))) {

						Map map = new HashMap();
						contained.put(pack, map);
						Resource pinfo = jar.getResource(prefix
								+ pack.replace('.', '/') + "/packageinfo");
						if (pinfo != null) {
							InputStream in = pinfo.openInputStream();
							String version = parsePackageInfo(in);
							in.close();
							if (version != null)
								map.put("version", version);
						}
					}
				}

				if (path.endsWith(".class")) {
					Resource resource = jar.getResource(path);
					Clazz clazz;

					try {
						InputStream in = resource.openInputStream();
						clazz = new Clazz(relativePath, in);
						in.close();
					} catch (Throwable e) {
						errors.add("Invalid class file: " + relativePath + " "
								+ e.getMessage());
						continue next;
					}
					classSpace.put(relativePath, clazz);
					referred.putAll(clazz.getReferred());

					// Add all the used packages
					// to this package
					Set t = (Set) uses.get(pack);
					if (t == null)
						uses.put(pack, t = new HashSet());
					t.addAll(clazz.getReferred().keySet());
					t.remove(pack);
				}
			}
		}
	}

	public String getPackage(String clazz) {
		int n = clazz.lastIndexOf('/');
		if (n < 0)
			return ".";
		return clazz.substring(0, n).replace('/', '.');
	}

	static Pattern	packageinfo	= Pattern.compile("version\\s+([0-9.]+).*");

	static String parsePackageInfo(InputStream jar) throws IOException {
		try {
			byte[] buf = EmbeddedResource.collect(jar, 0);
			String line = new String(buf).trim();
			Matcher m = packageinfo.matcher(line);
			if (m.matches()) {
				return m.group(1);
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		return null;
	}

	static Map removeKeys(Map source, String prefix) {
		Map temp = new TreeMap(source);
		for (Iterator p = temp.keySet().iterator(); p.hasNext();) {
			String pack = (String) p.next();
			if (pack.startsWith(prefix))
				p.remove();
		}
		return temp;
	}

	public List getProgress() {
		return progress;
	}

	public void progress(String s) {
		progress.add(s);
	}

	public boolean isPedantic() {
		return pedantic;
	}

	public void setPedantic(boolean pedantic) {		//System.out.println("Set pedantic: " + pedantic + "  " + this );
		this.pedantic = pedantic;
	}

	public static File getFile(File base, String file) {
		File f = new File(file);
		if (f.isAbsolute())
			return f;
		int n;

		f = base.getAbsoluteFile();
		while ((n = file.indexOf('/')) > 0) {
			String first = file.substring(0, n);
			file = file.substring(n + 1);
			if (first.equals(".."))
				f = f.getParentFile();
			else
				f = new File(f, first);
		}
		return new File(f, file);
	}

}

==> bndlib-0.0.198.jar: OSGI-OPT/src/aQute/lib/osgi/Processor.java <==
/* Copyright 2006 aQute SARL 
 * Licensed under the Apache License, Version 2.0, see http://www.apache.org/licenses/LICENSE-2.0 */
package aQute.lib.osgi;

import java.io.*;
import java.lang.reflect.*;
import java.util.*;
import java.util.regex.*;

import aQute.lib.osgi.header.*;
import aQute.lib.reporter.*;

public class Processor implements Reporter {
	public final static String	DEFAULT_BND_EXTENSION	= ".bnd";
	public final static String	DEFAULT_JAR_EXTENSION	= ".jar";
	public final static String	DEFAULT_BAR_EXTENSION	= ".bar";

	List						errors					= new ArrayList();
	List						warnings				= new ArrayList();
	List						progress				= new ArrayList();
	boolean						pedantic;

	String[]					METAPACKAGES			= { "META-INF",
			"OSGI-INF", "OSGI-OPT"						};

	public void getInfo(Reporter processor) {
		errors.addAll(processor.getErrors());
		warnings.addAll(processor.getWarnings());
		progress.addAll(processor.getProgress());
		pedantic = processor.isPedantic();
	}

	public void warning(String string) {
		warnings.add(string);
	}

	public void error(String string) {
		errors.add(string);
	}

	public List getWarnings() {
		return warnings;
	}

	public List getErrors() {
		return errors;
	}

	public Map parseHeader(String value) {
		return parseHeader(value, this);
	}

	/**
	 * Standard OSGi header parser.
	 * 
	 * @param value
	 * @return
	 */
	static public Map parseHeader(String value, Processor logger) {
		return OSGiHeader.parseHeader(value, logger);
	}

	public Map analyzeBundleClasspath(Jar dot, Map bundleClasspath,
			Map contained, Map referred, Map uses) throws IOException {
		Map classSpace = new HashMap();

		if (bundleClasspath.isEmpty()) {
			analyzeJar(dot, "", classSpace, contained, referred, uses);
		} else {
			for (Iterator j = bundleClasspath.keySet().iterator(); j.hasNext();) {
				String path = (String) j.next();
				if (path.equals(".")) {
					analyzeJar(dot, "", classSpace, contained, referred, uses);
					continue;
				}
				//
				// There are 3 cases:
				// - embedded JAR file
				// - directory
				// - error
				//

				Resource resource = dot.getResource(path);
				if (resource != null) {
					try {
						Jar jar = new Jar(path);
						EmbeddedResource.build(jar, resource);
						analyzeJar(jar, "", classSpace, contained, referred,
								uses);
					} catch (Exception e) {
						warning("Invalid bundle classpath entry: " + path +
								" " + e);
					}
				} else {
					if (dot.getDirectories().containsKey(path)) {
						analyzeJar(dot, path, classSpace, contained, referred,
								uses);
					} else {
						warning("No sub JAR or directory " + path);
					}
				}
			}
		}
		return classSpace;
	}

	/**
	 * We traverse through all the classes that we can find and calculate the
	 * contained and referred set and uses. This method ignores the Bundle
	 * classpath.
	 * 
	 * @param jar
	 * @param contained
	 * @param referred
	 * @param uses
	 * @throws IOException
	 */
	private void analyzeJar(Jar jar, String prefix, Map classSpace,
			Map contained, Map referred, Map uses) throws IOException {
		next: for (Iterator r = jar.getResources().keySet().iterator(); r
				.hasNext();) {
			String path = (String) r.next();
			if (path.startsWith(prefix)) {
				String relativePath = path.substring(prefix.length());
				String pack = getPackage(relativePath);

				if (pack != null && !contained.containsKey(pack)) {
					if (!(pack.equals(".") || isMetaData(relativePath))) {

						Map map = new HashMap();
						contained.put(pack, map);
						Resource pinfo = jar.getResource(prefix +
								pack.replace('.', '/') + "/packageinfo");
						if (pinfo != null) {
							InputStream in = pinfo.openInputStream();
							String version = parsePackageInfo(in);
							in.close();
							if (version != null)
								map.put("version", version);
						}
					}
				}

				if (path.endsWith(".class")) {
					Resource resource = jar.getResource(path);
					Clazz clazz;

					try {
						InputStream in = resource.openInputStream();
						clazz = new Clazz(relativePath, in);
						in.close();
					} catch (Throwable e) {
						errors.add("Invalid class file: " + relativePath + " " +
								e.getMessage());
						continue next;
					}
					classSpace.put(relativePath, clazz);
					referred.putAll(clazz.getReferred());

					// Add all the used packages
					// to this package
					Set t = (Set) uses.get(pack);
					if (t == null)
						uses.put(pack, t = new HashSet());
					t.addAll(clazz.getReferred().keySet());
					t.remove(pack);
				}
			}
		}
	}

	/**
	 * Decide if the package is a metadata package.
	 * 
	 * @param pack
	 * @return
	 */
	boolean isMetaData(String pack) {
		for (int i = 0; i < METAPACKAGES.length; i++) {
			if (pack.startsWith(METAPACKAGES[i]))
				return true;
		}
		return false;
	}

	public String getPackage(String clazz) {
		int n = clazz.lastIndexOf('/');
		if (n < 0)
			return ".";
		return clazz.substring(0, n).replace('/', '.');
	}

	static Pattern	packageinfo	= Pattern.compile("version\\s+([0-9.]+).*");

	static String parsePackageInfo(InputStream jar) throws IOException {
		try {
			byte[] buf = EmbeddedResource.collect(jar, 0);
			String line = new String(buf).trim();
			Matcher m = packageinfo.matcher(line);
			if (m.matches()) {
				return m.group(1);
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		return null;
	}

	static Map removeKeys(Map source, String prefix) {
		Map temp = new TreeMap(source);
		for (Iterator p = temp.keySet().iterator(); p.hasNext();) {
			String pack = (String) p.next();
			if (pack.startsWith(prefix))
				p.remove();
		}
		return temp;
	}

	public List getProgress() {
		return progress;
	}

	public void progress(String s) {
		progress.add(s);
	}

	public boolean isPedantic() {
		return pedantic;
	}

	public void setPedantic(boolean pedantic) { // System.out.println("Set
												// pedantic: " + pedantic + " "
												// + this );
		this.pedantic = pedantic;
	}

	public static File getFile(File base, String file) {
		File f = new File(file);
		if (f.isAbsolute())
			return f;
		int n;

		f = base.getAbsoluteFile();
		while ((n = file.indexOf('/')) > 0) {
			String first = file.substring(0, n);
			file = file.substring(n + 1);
			if (first.equals(".."))
				f = f.getParentFile();
			else
				f = new File(f, first);
		}
		return new File(f, file);
	}

	public void report(PrintWriter pw) {
		pw.println(getClass().getName());
		Class c = getClass();
		while ( c != Object.class ) {
			Field fields[] = c.getDeclaredFields();
			for(int i=0; i<fields.length; i++ ) {
				Field field = fields[i];
				field.setAccessible(true);
				try {
				format(pw, field.getName(), field.get(this));
				} catch( Exception e ) {
					format(pw, field.getName(), e );
				}
			}
		}
	}

	public void format(PrintWriter pw, String header, Object value) {
		pw.print(header);
		for (int i = header.length(); i < 30; i++)
			pw.print(" ");
		pw.println(value);
	}

}
